unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Math, registry, NoTask;

const


{
  ----- parts from the VirtuaWin - SDK ------- start

 * This is a definition of all possible messages to send to VirtuaWin
 * and the messages that VirtuaWin uses for module communication
 *
 * Created by Johan Piculell 1999
 * VirtuaWin@home.se
 * Feel free to use in any way you like

 * All messages starting with VW_ is for controlling VirtuaWin
 * and messages starting with MOD_ is messages sent by VirtuaWin
 *
 * For example if you want to step one desktop to the left:
 * PostMessage(VirtuaWin HWND, VW_CHANGEDESK, VW_STEPLEFT, 0);
 * For messages where you expect a return value, use the SendMessage
 * function instead, see some win32 documentation for more info.}


// Message, switch to a specified desktop, sent with following lParam or 1..9
  VW_CHANGEDESK =(WM_USER + 10);
  VW_STEPLEFT   =(WM_USER + 11); // lParam
  VW_STEPRIGHT  =(WM_USER + 12); // lParam
  VW_STEPUP     =(WM_USER + 13); // lParam
  VW_STEPDOWN   =(WM_USER + 14); // lParam
  { Message, close VirtuaWin }
  VW_CLOSE      =(WM_USER + 15);
  { Message, display setup dialog }
  VW_SETUP      =(WM_USER + 16);
  { Message, gather all windows }
  VW_GATHER     =(WM_USER + 19);
  { Message, retuns desktop width }
  VW_DESKX      =(WM_USER + 20);
  { Message, retuns desktop height }
  VW_DESKY      =(WM_USER + 21);
  VW_CURDESK    =(WM_USER + 23);

  { Message, sent by VirtuaWin after a switch. lParam will contain current desktop number
     if wParam isn't one of the following, then wParam will also contain current desktop.
     If desktop cycling is enabled, there will be two MOD_CHANGEDESK sent when going
     "over the edge". The first one will have a MOD_STEP* parameter, and the second
     will have current desktop in both parameters.
  }
  MOD_CHANGEDESK =(WM_USER + 30);
  MOD_STEPLEFT   =(WM_USER + 31); // wParam
  MOD_STEPRIGHT  =(WM_USER + 32); // wParam
  MOD_STEPUP     =(WM_USER + 33); // wParam
  MOD_STEPDOWN   =(WM_USER + 34); // wParam

  MOD_INIT      =(WM_USER + 35);
  MOD_QUIT      =(WM_USER + 36);
  MOD_SETUP     =(WM_USER + 37);
  { Message, sent by VirtuaWin when the configuration has been updated }
  MOD_CFGCHANGE =(WM_USER + 38);
{  ----- from the VirtuaWin  - SDK ------- end}

  type
  TCopyData = record
      dwData: LongInt;
      cbData: LongInt;
      lpData: Pointer;
  end;
  PCopyData = ^TCopyData;     { for the WM_COPYDATA message}

  TForm1 = class(TForm)
    NoTask1: TNoTask;
    Memo1: TMemo;
    Label1: TLabel;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Memo2: TMemo;
    Button4: TButton;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
  private
  public
    procedure empfang(var m:TMessage);message WM_COPYDATA;  // also used with VW_WINLIST but not used in this application
    procedure setup(var m:TMessage);  message mod_setup;
    procedure start(var m:TMessage);  message mod_init;
    procedure stop (var m:TMessage);  message mod_quit;
    procedure change(var m:TMessage); message mod_changedesk;
    function getdesk: integer;
    function getdx: integer;
    function getdy: integer;
    function getdesks : integer;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
  end;

var
  Form1: TForm1;
  vwhandle : THandle;

procedure TraceString(Msg: String);

implementation
{$R *.DFM}

  procedure TForm1.CreateParams(var Params: TCreateParams);
  {this is necessary, because VW detects the plugin by the windowclass name
   important: this is not equal to the "delphi-window-class-name" (in this case TForm1)}
  var a : pchar;
  begin
    inherited CreateParams(Params);
    a := pchar(ExtractFileName(ParamStr(0)));
    StrLCopy(Params.WinClassName,a,sizeof(Params.WinClassName));
  end;

  procedure TForm1.empfang(var m:TMessage);
  var C: string;
      int1 : integer;
  begin
    c := string(TCopyData(ptr(m.lParam)^).lpdata);
    label1.caption := c;
    case c[1] of
      '1'..'9' :begin
                  int1 := min(strtoint(c[1]),getdx*getdy);
                  sendmessage(vwhandle,VW_CHANGEDESK,int1,0);
                end;
      '+' : sendmessage(vwhandle,VW_CHANGEDESK,(getdesk+strtoint(c[2])-1) mod getdesks+1,0);
      '-' : sendmessage(vwhandle,VW_CHANGEDESK,(getdesk-strtoint(c[2])+getdesks*10-1) mod getdesks+1,0);
      'g','G' : sendmessage(vwhandle,VW_GATHER,0,0);
      'd','D' : sendmessage(vwhandle,vw_changedesk,0,0); {disables VW - undocumented?}
    else
      showmodal;
    end;
  end;

  procedure TForm1.setup(var m:TMessage);
  var reg : TRegistry;
  begin
    NoTask1.hideform := false;
    reg := TRegistry.Create;
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    if not reg.KeyExists('\HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\vwp.EXE') then
      button4.caption := 'Uninstall'
    else
      button4.caption := 'Install';
    application.bringtofront;
    showmodal;
    NoTask1.hideform := true;
    reg.closekey;
    reg.free;
  end;
  procedure TForm1.start(var m:TMessage);
  begin
    Visible := false;
    NoTask1.hideform := true;
    vwhandle :=  m.wparam;
  end;
  procedure TForm1.stop(var m:TMessage);
  begin
    close;
  end;
  procedure TForm1.change(var m:TMessage);
  begin
  end;
  function TForm1.getdesk : integer;
  begin
    result := sendmessage(vwhandle,vw_Curdesk,0,0);
  end;
  function TForm1.getdx: integer;
  begin
    result := sendmessage(vwhandle,vw_DESKX,0,0);
  end;
  function TForm1.getdy: integer;
  begin
    result := sendmessage(vwhandle,vw_DESKY,0,0);
  end;
  function TForm1.getdesks : integer;
  begin
    result := getdx*getdy;
  end;


procedure TraceString(Msg: String);
 {this sends an string to the another running instance}
var
   CopyData: TCopyData;
   TraceWin: hWnd;
   Buf: array[0..255] of Char;
begin
   TraceWin := FindWindow(pchar(ExtractFileName(ParamStr(0))), 'VirtuaWinPlugin');
   if TraceWin = 0 then
        exit;
   if Length(Msg) > 255 then
      SetLength(Msg, 255);
   StrPCopy(Buf, Msg);
   CopyData.dwData := 0;
   CopyData.cbData := Length(Msg) + 1;
   CopyData.lpData := @Buf[0];
   SendMessage(TraceWin, WM_COPYDATA, 0, LongInt(@CopyData));
end;


procedure TForm1.Button1Click(Sender: TObject);
begin
  if vwhandle = 0 then close;   // 0 if not called by virtuaWin
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  memo2.visible := false;
  memo1.visible := true;
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  memo1.visible := false;
  memo2.visible := true;
end;

procedure TForm1.Button4Click(Sender: TObject);
var reg :TRegistry;
begin
  reg := TRegistry.Create;
  reg.lazywrite := false;
  Reg.RootKey := HKEY_LOCAL_MACHINE;
  if button4.caption = 'Uninstall' then begin
    reg.openkey('\SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\',false);
    reg.DeleteKey('vwp.EXE');
    button4.caption := 'Install';
  end else begin
    if reg.openkey('\SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\vwp.EXE',true) then
      reg.WriteString('',ExtractFilePath(ParamStr(0)));
    button4.caption := 'Uninstall';
  end;
  Reg.CloseKey;
  Reg.Free;
end;

end.
